@@ -1,6 +1,6 @@
 #' Control How Progress is Reported
 #'
-#' @param \dots One or more progression handlers.  Alternatively, this
+#' @param \ldots One or more progression handlers.  Alternatively, this
 #' functions accepts also a single vector of progression handlers as input.
 #' If this vector is empty, then an empty set of progression handlers will
 #' be set.
@@ -109,6 +109,7 @@ handlers <- function(..., append = FALSE, on_missing = c("error", "warning", "ig
   }
 
   handlers <- list()
+  
   names <- names(args)
   for (kk in seq_along(args)) {
     handler <- args[[kk]]
@@ -137,17 +138,26 @@ handlers <- function(..., append = FALSE, on_missing = c("error", "warning", "ig
 	}
         next
       }
-
-      names[kk] <- name
+    } else {
+      name <- NULL
     }
     stop_if_not(is.function(handler), length(formals(handler)) >= 1L)
+
+    ## Validate?
+    validator <- attr(handler, "validator")
+    if (is.function(validator)) {
+      is_valid <- validator()
+      if (!is_valid) next
+    }
+
+    if (!is.null(name)) names[kk] <- name
     handlers[[kk]] <- handler
   }
   stop_if_not(is.list(handlers))
   names(handlers) <- names
 
   ## Drop non-existing handlers
-  keep <- vapply(handlers, FUN = is.function, FUN.VALUE = FALSE)
+  keep <- vapply(handlers, FUN.VALUE = FALSE, FUN = is.function)
   handlers <- handlers[keep]
 
   if (append) {
@@ -155,5 +165,8 @@ handlers <- function(..., append = FALSE, on_missing = c("error", "warning", "ig
     if (length(current) > 0L) handlers <- c(current, handlers)
   }
 
-  invisible(options(progressr.handlers = handlers)[[1]])
+  old_handlers <- options(progressr.handlers = handlers)[[1]]
+  if (is.null(old_handlers)) old_handlers <- list()
+  
+  invisible(old_handlers)
 }
